#' @title Plot the paired measurements
#'
#' @description Plot a scatter plot of the paired measurements---the second
#' measurement against the first measurement.
#'
#' @param x
#' An object of class "PM_Evaluation"
#'
#' @param ...
#' Required for compatibility with generic definition
#'
#' @return A ggplot2 object to implement the previously-described plotting.
#'
#' @examples
#' oEvaluation <- PM_Evaluation(ExampleData, "Example data (no units)")
#' plot(oEvaluation)
#'
#' @export
#'
plot.PM_Evaluation <- function(x, ...) {

  ggplot2::ggplot(x$measurements) +
    ggplot2::geom_abline(slope = 1, color = "red") +
    ggplot2::geom_point(ggplot2::aes(x = first, y = second), color = "black") +
    ggplot2::scale_x_continuous(name = paste0("First measurement from ", x$axisLabel),
                                limits = x$theRange) +
    ggplot2::scale_y_continuous(name = paste0("Second measurement from ", x$axisLabel),
                                limits = x$theRange) +
    ggplot2::coord_equal() +
    ggplot2::theme_bw() +
    ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0,face = "bold.italic", size = 10)) +
    ggplot2::theme(panel.grid.major = ggplot2::element_blank(), panel.grid.minor = ggplot2::element_blank())

}

#' @title Summarize the paired measurements
#'
#' @description Print summary statistics for the paired measurements.
#'
#' @param object
#' An object of class "PM_Evaluation"
#'
#' @param ...
#' Required for compatibility with generic definition
#'
#' @examples
#' oEvaluation <- PM_Evaluation(ExampleData, "Example data (no units)")
#' summary(oEvaluation)
#'
#' @export
#'
summary.PM_Evaluation <- function(object, ...) {

  cat(sprintf( "###############################################################\n"))
  cat(sprintf("Summary\n"))
  cat(sprintf("Number of paired measurements: %g\n", nrow(object$measurements)))
  cat(sprintf("Range: %g   %g\n", object$theRange[1], object$theRange[2]))
  cat(sprintf("Standard deviation of the paired measurements: %g\n", object$stdDev))
  cat(sprintf( "###############################################################\n"))

}

#' @title Constructor of class that evaluates paired measurements.
#'
#' @description
#' This constructor stores the paired measurements and
#' calculates some quantities that are needed to evaluate
#' the paired measurements.
#'
#' @param measurements
#' R-language dataframe containing the paired measurements
#' Each row pertains to one
#' pair of measurements. Column \code{first} contains the first measurement for
#' every pair, and column \code{second} contains the second measurement for
#' every pair.
#'
#' @param axisLabel
#' Character string describing the measurements. This string is used as
#' a label in the plots.
#'
#' @return A list with the following components is returned.
#' @return \item{measurements}{Argument measurements.}
#' @return \item{axisLabel}{Argument axisLabel.}
#' @return \item{theRange}{Range of the measurements.}
#' @return \item{stdDev}{Standard deviation of the paired measurements.}
#' @return \item{call}{Function call.}
#'
#' @examples
#' oEvaluation <- PM_Evaluation(ExampleData, "Example data (no units)")
#'
#' @export
#'
PM_Evaluation <- function(measurements, axisLabel) {

  d <- measurements$first - measurements$second
  stdDev <- sqrt(0.5 / length(d) * sum(d^2))

  rval <- list(measurements = measurements,
               axisLabel = axisLabel,
               theRange = range(measurements[, c("first", "second")]),
               stdDev = stdDev,
               call = sys.call())

  class(rval) <- "PM_Evaluation"

  return(rval)
}
